@@ -82,6 +82,27 @@ _ARRAY_EXTEND_ARGS = value_args_model([
 ])
 
 
+# $function: arrayFlat
+# $group: Array
+# $doc: Flat an array hierarchy
+# $arg array: The array to flat
+# $return: The flated array
+def _array_flat(args, unused_options):
+    array, = value_args_validate(_ARRAY_FLAT_ARGS, args)
+    return list(_array_flat_helper(array))
+
+def _array_flat_helper(array):
+    for item in array:
+        if isinstance(item, list):
+            yield from _array_flat_helper(item)
+        else:
+            yield item
+
+_ARRAY_FLAT_ARGS = value_args_model([
+    {'name': 'array', 'type': 'array'}
+])
+
+
 # $function: arrayGet
 # $group: Array
 # $doc: Get an array element
@@ -340,6 +361,53 @@ _ARRAY_SORT_ARGS = value_args_model([
 ])
 
 
+#
+# Coverage functions
+#
+
+
+# Coverage configuration object global variable name
+COVERAGE_GLOBAL_NAME = '__bareScriptCoverage'
+
+
+# $function: coverageGlobalGet
+# $group: Coverage
+# $doc: Get the coverage global object
+# $return: The [coverage global object](https://craigahobbs.github.io/bare-script/model/#var.vName='CoverageGlobal')
+def _coverage_global_get(unused_args, options):
+    globals_ = options.get('globals') if options is not None else None
+    return globals_.get(COVERAGE_GLOBAL_NAME) if globals_ is not None else None
+
+
+# $function: coverageGlobalName
+# $group: Coverage
+# $doc: Get the coverage global variable name
+# $return: The coverage global variable name
+def _coverage_global_name(unused_args, unused_options):
+    return COVERAGE_GLOBAL_NAME
+
+
+# $function: coverageStart
+# $group: Coverage
+# $doc: Start coverage data collection
+def _coverage_start(unused_args, options):
+    globals_ = options.get('globals') if options is not None else None
+    if globals_ is not None:
+        coverage_global = {'enabled': True}
+        globals_[COVERAGE_GLOBAL_NAME] = coverage_global
+
+
+# $function: coverageStop
+# $group: Coverage
+# $doc: Stop coverage data collection
+def _coverage_stop(unused_args, options):
+    globals_ = options.get('globals') if options is not None else None
+    if globals_ is not None:
+        coverage_global = globals_.get(COVERAGE_GLOBAL_NAME)
+        if coverage_global is not None:
+            globals_[COVERAGE_GLOBAL_NAME]['enabled'] = False
+
+
 #
 # Data functions
 #
@@ -2008,6 +2076,7 @@ SCRIPT_FUNCTIONS = {
     'arrayCopy': _array_copy,
     'arrayDelete': _array_delete,
     'arrayExtend': _array_extend,
+    'arrayFlat': _array_flat,
     'arrayGet': _array_get,
     'arrayIndexOf': _array_index_of,
     'arrayJoin': _array_join,
@@ -2021,6 +2090,10 @@ SCRIPT_FUNCTIONS = {
     'arrayShift': _array_shift,
     'arraySlice': _array_slice,
     'arraySort': _array_sort,
+    'coverageGlobalGet': _coverage_global_get,
+    'coverageGlobalName': _coverage_global_name,
+    'coverageStart': _coverage_start,
+    'coverageStop': _coverage_stop,
     'dataAggregate': _data_aggregate,
     'dataCalculatedField': _data_calculated_field,
     'dataFilter': _data_filter,
@@ -2118,6 +2191,7 @@ SCRIPT_FUNCTIONS = {
 EXPRESSION_FUNCTION_MAP = {
     'abs': 'mathAbs',
     'acos': 'mathAcos',
+    'arrayNew': 'arrayNew',
     'asin': 'mathAsin',
     'atan': 'mathAtan',
     'atan2': 'mathAtan2',
@@ -2143,6 +2217,7 @@ EXPRESSION_FUNCTION_MAP = {
     'minute': 'datetimeMinute',
     'month': 'datetimeMonth',
     'now': 'datetimeNow',
+    'objectNew': 'objectNew',
     'parseInt': 'numberParseInt',
     'parseFloat': 'numberParseFloat',
     'pi': 'mathPi',
