@@ -8,7 +8,7 @@ The BareScript language parser
 import re
 
 
-def parse_script(script_text, start_line_number=1):
+def parse_script(script_text, start_line_number=1, script_name=None):
     """
     Parse a BareScript script
 
@@ -16,15 +16,19 @@ def parse_script(script_text, start_line_number=1):
     :type script_text: str or ~collections.abc.Iterable(str)
     :param start_line_number: The script's starting line number
     :type start_line_number: int, optional
+    :param script_name: The script name
+    :type script_name: str or None, optional
     :return: The `BareScript model <./model/#var.vName='BareScript'>`__
     :rtype: dict
     :raises BareScriptParserError: A parsing error occurred
     """
 
-    script = {'statements': []}
+    lines = []
+    script = {'statements': [], 'scriptLines': lines}
+    if script_name is not None:
+        script['scriptName'] = script_name
 
     # Line-split all script text
-    lines = []
     if isinstance(script_text, str):
         lines.extend(_R_SCRIPT_LINE_SPLIT.split(script_text))
     else:
@@ -64,6 +68,12 @@ def parse_script(script_text, start_line_number=1):
         else:
             line = line_part
 
+        # Base statement members
+        line_number = ix_line + 1
+        statement_base = {'lineNumber': line_number}
+        if ix_line != ix_line_part:
+            statement_base['lineCount'] = (ix_line_part - ix_line) + 1
+
         # Assignment?
         match_assignment = _R_SCRIPT_ASSIGNMENT.match(line)
         if match_assignment:
@@ -71,28 +81,30 @@ def parse_script(script_text, start_line_number=1):
                 expr_statement = {
                     'expr': {
                         'name': match_assignment.group('name'),
-                        'expr': parse_expression(match_assignment.group('expr'))
+                        'expr': parse_expression(match_assignment.group('expr'), line_number, script_name, True),
+                        **statement_base
                     }
                 }
                 statements.append(expr_statement)
                 continue
             except BareScriptParserError as error:
                 column_number = len(line) - len(match_assignment.group('expr')) + error.column_number
-                raise BareScriptParserError(error.error, line, column_number, start_line_number + ix_line)
+                raise BareScriptParserError(error.error, line, column_number, start_line_number + ix_line, script_name)
 
         # Function definition begin?
         match_function_begin = _R_SCRIPT_FUNCTION_BEGIN.match(line)
         if match_function_begin:
             # Nested function definitions are not allowed
             if function_def is not None:
-                raise BareScriptParserError('Nested function definition', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('Nested function definition', line, 1, start_line_number + ix_line, script_name)
 
             # Add the function definition statement
             function_label_def_depth = len(label_defs)
             function_def = {
                 'function': {
                     'name': match_function_begin.group('name'),
-                    'statements': []
+                    'statements': [],
+                    **statement_base
                 }
             }
             if match_function_begin.group('args') is not None:
@@ -108,14 +120,14 @@ def parse_script(script_text, start_line_number=1):
         match_function_end = _R_SCRIPT_FUNCTION_END.match(line)
         if match_function_end:
             if function_def is None:
-                raise BareScriptParserError('No matching function definition', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('No matching function definition', line, 1, start_line_number + ix_line, script_name)
 
             # Check for un-matched label definitions
             if len(label_defs) > function_label_def_depth:
                 label_def = label_defs.pop()
                 def_key = next(iter(label_def))
                 def_ = label_def[def_key]
-                raise BareScriptParserError(f"Missing end{def_key} statement", def_['line'], 1, def_['lineNumber'])
+                raise BareScriptParserError(f"Missing end{def_key} statement", def_['line'], 1, def_['lineNumber'], script_name)
 
             function_def = None
             function_label_def_depth = None
@@ -128,7 +140,8 @@ def parse_script(script_text, start_line_number=1):
             ifthen = {
                 'jump': {
                     'label': f"__bareScriptIf{label_index}",
-                    'expr': {'unary': {'op': '!', 'expr': parse_expression(match_if_begin.group('expr'))}}
+                    'expr': {'unary': {'op': '!', 'expr': parse_expression(match_if_begin.group('expr'), line_number, script_name, True)}},
+                    **statement_base
                 },
                 'done': f"__bareScriptDone{label_index}",
                 'hasElse': False,
@@ -149,24 +162,25 @@ def parse_script(script_text, start_line_number=1):
             label_def_depth = function_label_def_depth if function_def is not None else 0
             ifthen = label_defs[len(label_defs) - 1].get('if') if len(label_defs) > label_def_depth else None
             if ifthen is None:
-                raise BareScriptParserError('No matching if statement', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('No matching if statement', line, 1, start_line_number + ix_line, script_name)
 
             # Cannot come after the else-then statement
             if ifthen['hasElse']:
-                raise BareScriptParserError('Elif statement following else statement', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('Elif statement following else statement', line, 1, start_line_number + ix_line, script_name)
 
             # Generate the next if-then jump statement
             prev_label = ifthen['jump']['label']
             ifthen['jump'] = {
                 'label': f"__bareScriptIf{label_index}",
-                'expr': {'unary': {'op': '!', 'expr': parse_expression(match_if_else_if.group('expr'))}}
+                'expr': {'unary': {'op': '!', 'expr': parse_expression(match_if_else_if.group('expr'), line_number, script_name, True)}},
+                **statement_base
             }
             label_index += 1
 
             # Add the if-then else statements
             statements.extend([
-                {'jump': {'label': ifthen['done']}},
-                {'label': prev_label},
+                {'jump': {'label': ifthen['done'], **statement_base}},
+                {'label': {'name': prev_label, **statement_base}},
                 {'jump': ifthen['jump']}
             ])
             continue
@@ -178,17 +192,17 @@ def parse_script(script_text, start_line_number=1):
             label_def_depth = function_label_def_depth if function_def is not None else 0
             ifthen = label_defs[len(label_defs) - 1].get('if') if len(label_defs) > label_def_depth else None
             if ifthen is None:
-                raise BareScriptParserError('No matching if statement', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('No matching if statement', line, 1, start_line_number + ix_line, script_name)
 
             # Cannot have multiple else-then statements
             if ifthen['hasElse']:
-                raise BareScriptParserError('Multiple else statements', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('Multiple else statements', line, 1, start_line_number + ix_line, script_name)
             ifthen['hasElse'] = True
 
             # Add the if-then else statements
             statements.extend([
-                {'jump': {'label': ifthen['done']}},
-                {'label': ifthen['jump']['label']}
+                {'jump': {'label': ifthen['done'], **statement_base}},
+                {'label': {'name': ifthen['jump']['label'], **statement_base}}
             ])
             continue
 
@@ -199,14 +213,14 @@ def parse_script(script_text, start_line_number=1):
             label_def_depth = function_label_def_depth if function_def is not None else 0
             ifthen = label_defs.pop().get('if') if len(label_defs) > label_def_depth else None
             if ifthen is None:
-                raise BareScriptParserError('No matching if statement', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('No matching if statement', line, 1, start_line_number + ix_line, script_name)
 
             # Update the previous jump statement's label, if necessary
             if not ifthen['hasElse']:
                 ifthen['jump']['label'] = ifthen['done']
 
             # Add the if-then footer statement
-            statements.append({'label': ifthen['done']})
+            statements.append({'label': {'name': ifthen['done'], **statement_base}})
             continue
 
         # While-do begin?
@@ -217,7 +231,7 @@ def parse_script(script_text, start_line_number=1):
                 'loop': f'__bareScriptLoop{label_index}',
                 'continue': f'__bareScriptLoop{label_index}',
                 'done': f'__bareScriptDone{label_index}',
-                'expr': parse_expression(match_while_begin.group('expr')),
+                'expr': parse_expression(match_while_begin.group('expr'), line_number, script_name, True),
                 'line': line,
                 'lineNumber': start_line_number + ix_line
             }
@@ -225,8 +239,10 @@ def parse_script(script_text, start_line_number=1):
             label_index += 1
 
             # Add the while-do header statements
-            statements.append({'jump': {'label': whiledo['done'], 'expr': {'unary': {'op': '!', 'expr': whiledo['expr']}}}})
-            statements.append({'label': whiledo['loop']})
+            statements.extend([
+                {'jump': {'label': whiledo['done'], 'expr': {'unary': {'op': '!', 'expr': whiledo['expr']}}, **statement_base}},
+                {'label': {'name': whiledo['loop'], **statement_base}}
+            ])
             continue
 
         # While-do end?
@@ -235,15 +251,17 @@ def parse_script(script_text, start_line_number=1):
             # Pop the while-do definition
             label_def_depth = function_label_def_depth if function_def is not None else 0
             if len(label_defs) <= label_def_depth:
-                raise BareScriptParserError('No matching while statement', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('No matching while statement', line, 1, start_line_number + ix_line, script_name)
 
             whiledo = label_defs.pop().get('while')
             if not whiledo:
-                raise BareScriptParserError('No matching while statement', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('No matching while statement', line, 1, start_line_number + ix_line, script_name)
 
             # Add the while-do footer statements
-            statements.append({'jump': {'label': whiledo['loop'], 'expr': whiledo['expr']}})
-            statements.append({'label': whiledo['done']})
+            statements.extend([
+                {'jump': {'label': whiledo['loop'], 'expr': whiledo['expr'], **statement_base}},
+                {'label': {'name': whiledo['done'], **statement_base}}
+            ])
             continue
 
         # For-each begin?
@@ -266,17 +284,27 @@ def parse_script(script_text, start_line_number=1):
 
             # Add the for-each header statements
             statements.extend([
-                {'expr': {'name': foreach['values'], 'expr': parse_expression(match_for_begin.group('values'))}},
+                {'expr': {
+                    'name': foreach['values'],
+                    'expr': parse_expression(match_for_begin.group('values'), line_number, script_name, True),
+                    **statement_base
+                }},
                 {'expr': {
                     'name': foreach['length'],
-                    'expr': {'function': {'name': 'arrayLength', 'args': [{'variable': foreach['values']}]}}
+                    'expr': {'function': {'name': 'arrayLength', 'args': [{'variable': foreach['values']}]}},
+                    **statement_base
                 }},
-                {'jump': {'label': foreach['done'], 'expr': {'unary': {'op': '!', 'expr': {'variable': foreach['length']}}}}},
-                {'expr': {'name': foreach['index'], 'expr': {'number': 0}}},
-                {'label': foreach['loop']},
+                {'jump': {
+                    'label': foreach['done'],
+                    'expr': {'unary': {'op': '!', 'expr': {'variable': foreach['length']}}},
+                    **statement_base
+                }},
+                {'expr': {'name': foreach['index'], 'expr': {'number': 0}, **statement_base}},
+                {'label': {'name': foreach['loop'], **statement_base}},
                 {'expr': {
                     'name': foreach['value'],
-                    'expr': {'function': {'name': 'arrayGet', 'args': [{'variable': foreach['values']}, {'variable': foreach['index']}]}}
+                    'expr': {'function': {'name': 'arrayGet', 'args': [{'variable': foreach['values']}, {'variable': foreach['index']}]}},
+                    **statement_base
                 }}
             ])
             continue
@@ -287,25 +315,27 @@ def parse_script(script_text, start_line_number=1):
             # Pop the foreach definition
             label_def_depth = function_label_def_depth if function_def is not None else 0
             if len(label_defs) <= label_def_depth:
-                raise BareScriptParserError('No matching for statement', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('No matching for statement', line, 1, start_line_number + ix_line, script_name)
 
             foreach = label_defs.pop().get('for')
             if not foreach:
-                raise BareScriptParserError('No matching for statement', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('No matching for statement', line, 1, start_line_number + ix_line, script_name)
 
             # Add the for-each footer statements
             if foreach.get('hasContinue'):
-                statements.append({'label': foreach['continue']})
+                statements.append({'label': {'name': foreach['continue'], **statement_base}})
             statements.extend([
                 {'expr': {
                     'name': foreach['index'],
-                    'expr': {'binary': {'op': '+', 'left': {'variable': foreach['index']}, 'right': {'number': 1}}}
+                    'expr': {'binary': {'op': '+', 'left': {'variable': foreach['index']}, 'right': {'number': 1}}},
+                    **statement_base
                 }},
                 {'jump': {
                     'label': foreach['loop'],
-                    'expr': {'binary': {'op': '<', 'left': {'variable': foreach['index']}, 'right': {'variable': foreach['length']}}}
+                    'expr': {'binary': {'op': '<', 'left': {'variable': foreach['index']}, 'right': {'variable': foreach['length']}}},
+                    **statement_base
                 }},
-                {'label': foreach['done']}
+                {'label': {'name': foreach['done'], **statement_base}}
             ])
             continue
 
@@ -316,13 +346,13 @@ def parse_script(script_text, start_line_number=1):
             label_def_depth = function_label_def_depth if function_def is not None else 0
             ix_label_def = next((i for i, d in reversed(list(enumerate(label_defs))) if 'if' not in d), -1)
             if ix_label_def < label_def_depth:
-                raise BareScriptParserError('Break statement outside of loop', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('Break statement outside of loop', line, 1, start_line_number + ix_line, script_name)
             label_def = label_defs[ix_label_def]
             label_key = next(iter(label_def))
             loop_def = label_def[label_key]
 
             # Add the break jump statement
-            statements.append({'jump': {'label': loop_def['done']}})
+            statements.append({'jump': {'label': loop_def['done'], **statement_base}})
             continue
 
         # Continue statement?
@@ -332,45 +362,45 @@ def parse_script(script_text, start_line_number=1):
             label_def_depth = function_label_def_depth if function_def is not None else 0
             ix_label_def = next((i for i, d in reversed(list(enumerate(label_defs))) if 'if' not in d), -1)
             if ix_label_def < label_def_depth:
-                raise BareScriptParserError('Continue statement outside of loop', line, 1, start_line_number + ix_line)
+                raise BareScriptParserError('Continue statement outside of loop', line, 1, start_line_number + ix_line, script_name)
             label_def = label_defs[ix_label_def]
             label_key = next(iter(label_def))
             loop_def = label_def[label_key]
 
             # Add the continue jump statement
             loop_def['hasContinue'] = True
-            statements.append({'jump': {'label': loop_def['continue']}})
+            statements.append({'jump': {'label': loop_def['continue'], **statement_base}})
             continue
 
         # Label definition?
         match_label = _R_SCRIPT_LABEL.match(line)
         if match_label:
-            statements.append({'label': match_label.group('name')})
+            statements.append({'label': {'name': match_label.group('name'), **statement_base}})
             continue
 
         # Jump definition?
         match_jump = _R_SCRIPT_JUMP.match(line)
         if match_jump:
-            jump_statement = {'jump': {'label': match_jump.group('name')}}
+            jump_statement = {'jump': {'label': match_jump.group('name'), **statement_base}}
             if match_jump.group('expr'):
                 try:
-                    jump_statement['jump']['expr'] = parse_expression(match_jump.group('expr'))
+                    jump_statement['jump']['expr'] = parse_expression(match_jump.group('expr'), line_number, script_name, True)
                 except BareScriptParserError as error:
                     column_number = len(match_jump.group('jump')) - len(match_jump.group('expr')) - 1 + error.column_number
-                    raise BareScriptParserError(error.error, line, column_number, start_line_number + ix_line)
+                    raise BareScriptParserError(error.error, line, column_number, start_line_number + ix_line, script_name)
             statements.append(jump_statement)
             continue
 
         # Return definition?
         match_return = _R_SCRIPT_RETURN.match(line)
         if match_return:
-            return_statement = {'return': {}}
+            return_statement = {'return': {**statement_base}}
             if match_return.group('expr'):
                 try:
-                    return_statement['return']['expr'] = parse_expression(match_return.group('expr'))
+                    return_statement['return']['expr'] = parse_expression(match_return.group('expr'), line_number, script_name, True)
                 except BareScriptParserError as error:
                     column_number = len(match_return.group('return')) - len(match_return.group('expr')) + error.column_number
-                    raise BareScriptParserError(error.error, line, column_number, start_line_number + ix_line)
+                    raise BareScriptParserError(error.error, line, column_number, start_line_number + ix_line, script_name)
             statements.append(return_statement)
             continue
 
@@ -378,27 +408,30 @@ def parse_script(script_text, start_line_number=1):
         match_include = _R_SCRIPT_INCLUDE.match(line) or _R_SCRIPT_INCLUDE_SYSTEM.match(line)
         if match_include:
             delim = match_include.group('delim')
-            url = match_include.group('url') if delim == '<' else _R_EXPR_STRING_ESCAPE.sub('\\1', match_include.group('url'))
+            url = match_include.group('url') if delim == '<' else \
+                _R_EXPR_STRING_ESCAPES.sub(_replace_string_escape, match_include.group('url'))
             include_statement = statements[-1] if statements else None
             if include_statement is None or 'include' not in include_statement:
-                include_statement = {'include': {'includes': []}}
+                include_statement = {'include': {'includes': [], **statement_base}}
                 statements.append(include_statement)
+            else:
+                include_statement['include']['lineCount'] = (ix_line_part - include_statement['include']['lineNumber']) + 2
             include_statement['include']['includes'].append({'url': url, 'system': True} if delim == '<' else {'url': url})
             continue
 
         # Expression
         try:
-            expr_statement = {'expr': {'expr': parse_expression(line)}}
+            expr_statement = {'expr': {'expr': parse_expression(line, line_number, script_name, True), **statement_base}}
             statements.append(expr_statement)
         except BareScriptParserError as error:
-            raise BareScriptParserError(error.error, line, error.column_number, start_line_number + ix_line)
+            raise BareScriptParserError(error.error, line, error.column_number, start_line_number + ix_line, script_name)
 
     # Dangling label definitions?
     if label_defs:
         label_def = label_defs.pop()
         def_key = next(iter(label_def))
         def_ = label_def[def_key]
-        raise BareScriptParserError(f"Missing end{def_key} statement", def_['line'], 1, def_['lineNumber'])
+        raise BareScriptParserError(f"Missing end{def_key} statement", def_['line'], 1, def_['lineNumber'], script_name)
 
     return script
 
@@ -434,34 +467,40 @@ _R_SCRIPT_BREAK = re.compile(r'^\s*break' + _R_PART_COMMENT)
 _R_SCRIPT_CONTINUE = re.compile(r'^\s*continue' + _R_PART_COMMENT)
 
 
-def parse_expression(expr_text):
+def parse_expression(expr_text, line_number=None, script_name=None, array_literals=False):
     """
     Parse a BareScript expression
 
     :param expr_text: The `expression text <https://craigahobbs.github.io/bare-script/language/#expressions>`__
     :type expr_text: str or ~collections.abc.Iterable(str)
+    :param line_number: The script line number
+    :type line_number: int or None, optional
+    :param script_name: The script name
+    :type script_name: str or None, optional
+    :param array_literals: If True, allow parsing of array literals
+    :type array_literals: bool, optional
     :return: The `expression model <./model/#var.vName='Expression'>`__
     :rtype: dict
     :raises BareScriptParserError: A parsing error occurred
     """
     try:
-        expr, next_text = _parse_binary_expression(expr_text)
+        expr, next_text = _parse_binary_expression(expr_text, None, array_literals)
         if next_text.strip() != '':
-            raise BareScriptParserError('Syntax error', next_text)
+            raise BareScriptParserError('Syntax error', next_text, 1, line_number, script_name)
         return expr
     except BareScriptParserError as error:
         column_number = len(expr_text) - len(error.line) + 1
-        raise BareScriptParserError(error.error, expr_text, column_number)
+        raise BareScriptParserError(error.error, expr_text, column_number, line_number, script_name)
 
 
 # Helper function to parse a binary operator expression chain
-def _parse_binary_expression(expr_text, bin_left_expr=None):
+def _parse_binary_expression(expr_text, bin_left_expr, array_literals):
     # Parse the binary operator's left unary expression if none was passed
     if bin_left_expr is not None:
         bin_text = expr_text
         left_expr = bin_left_expr
     else:
-        left_expr, bin_text = _parse_unary_expression(expr_text)
+        left_expr, bin_text = _parse_unary_expression(expr_text, array_literals)
 
     # Match a binary operator - if not found, return the left expression
     match_binary_op = _R_EXPR_BINARY_OP.match(bin_text)
@@ -475,7 +514,7 @@ def _parse_binary_expression(expr_text, bin_left_expr=None):
     right_text = bin_text[len(match_binary_op.group(0)):]
 
     # Parse the right sub-expression
-    right_expr, next_text = _parse_unary_expression(right_text)
+    right_expr, next_text = _parse_unary_expression(right_text, array_literals)
 
     # Create the binary expression - re-order for binary operators as necessary
     bin_expr = None
@@ -490,7 +529,7 @@ def _parse_binary_expression(expr_text, bin_left_expr=None):
         bin_expr = {'binary': {'op': bin_op, 'left': left_expr, 'right': right_expr}}
 
     # Parse the next binary expression in the chain
-    return _parse_binary_expression(next_text, bin_expr)
+    return _parse_binary_expression(next_text, bin_expr, array_literals)
 
 
 # Binary operator re-order map
@@ -518,15 +557,15 @@ BINARY_REORDER = {
 
 
 # Helper function to parse a unary expression
-def _parse_unary_expression(expr_text):
+def _parse_unary_expression(expr_text, array_literals):
     # Group open?
     match_group_open = _R_EXPR_GROUP_OPEN.match(expr_text)
     if match_group_open:
         group_text = expr_text[len(match_group_open.group(0)):]
-        expr, next_text = _parse_binary_expression(group_text)
+        expr, next_text = _parse_binary_expression(group_text, None, array_literals)
         match_group_close = _R_EXPR_GROUP_CLOSE.match(next_text)
         if match_group_close is None:
-            raise BareScriptParserError('Unmatched parenthesis', expr_text)
+            raise BareScriptParserError('Unmatched parenthesis', expr_text, 1, None, None)
         return [{'group': expr}, next_text[len(match_group_close.group(0)):]]
 
     # Number?
@@ -540,14 +579,14 @@ def _parse_unary_expression(expr_text):
     # String?
     match_string = _R_EXPR_STRING.match(expr_text)
     if match_string:
-        string = _R_EXPR_STRING_ESCAPE.sub('\\1', match_string.group(1))
+        string = _R_EXPR_STRING_ESCAPES.sub(_replace_string_escape, match_string.group(1))
         expr = {'string': string}
         return [expr, expr_text[len(match_string.group(0)):]]
 
     # String (double quotes)?
     match_string_double = _R_EXPR_STRING_DOUBLE.match(expr_text)
     if match_string_double:
-        string = _R_EXPR_STRING_DOUBLE_ESCAPE.sub('\\1', match_string_double.group(1))
+        string = _R_EXPR_STRING_ESCAPES.sub(_replace_string_escape, match_string_double.group(1))
         expr = {'string': string}
         return [expr, expr_text[len(match_string_double.group(0)):]]
 
@@ -555,7 +594,7 @@ def _parse_unary_expression(expr_text):
     match_unary = _R_EXPR_UNARY_OP.match(expr_text)
     if match_unary:
         unary_text = expr_text[len(match_unary.group(0)):]
-        expr, next_text = _parse_unary_expression(unary_text)
+        expr, next_text = _parse_unary_expression(unary_text, array_literals)
         unary_expr = {'unary': {'op': match_unary.group(1), 'expr': expr}}
         return [unary_expr, next_text]
 
@@ -575,17 +614,80 @@ def _parse_unary_expression(expr_text):
             if args:
                 match_function_separator = _R_EXPR_FUNCTION_SEPARATOR.match(arg_text)
                 if match_function_separator is None:
-                    raise BareScriptParserError('Syntax error', arg_text)
+                    raise BareScriptParserError('Syntax error', arg_text, 1, None, None)
                 arg_text = arg_text[len(match_function_separator.group(0)):]
 
             # Get the argument
-            arg_expr, next_arg_text = _parse_binary_expression(arg_text)
+            arg_expr, next_arg_text = _parse_binary_expression(arg_text, None, array_literals)
             args.append(arg_expr)
             arg_text = next_arg_text
 
         fn_expr = {'function': {'name': match_function_open.group(1), 'args': args}}
         return [fn_expr, arg_text]
 
+    # Object creation?
+    match_object_open = _R_EXPR_OBJECT_OPEN.match(expr_text)
+    if match_object_open:
+        arg_text = expr_text[len(match_object_open.group(0)):]
+        args = []
+        while True:
+            # Object close?
+            match_object_close = _R_EXPR_OBJECT_CLOSE.match(arg_text)
+            if match_object_close:
+                arg_text = arg_text[len(match_object_close.group(0)):]
+                break
+
+            # Object key separator
+            if args:
+                match_object_key_separator = _R_EXPR_OBJECT_SEPARATOR2.match(arg_text)
+                if match_object_key_separator is None:
+                    raise BareScriptParserError('Syntax error', arg_text, 1, None, None)
+                arg_text = arg_text[len(match_object_key_separator.group(0)):]
+
+            # Get the key
+            arg_key, arg_text = _parse_binary_expression(arg_text, None, array_literals)
+            args.append(arg_key)
+
+            # Value separator
+            match_object_value_separator = _R_EXPR_OBJECT_SEPARATOR.match(arg_text)
+            if match_object_value_separator is None:
+                raise BareScriptParserError('Syntax error', arg_text, 1, None, None)
+            arg_text = arg_text[len(match_object_value_separator.group(0)):]
+
+            # Get the value
+            arg_value, arg_text = _parse_binary_expression(arg_text, None, array_literals)
+            args.append(arg_value)
+
+        fn_expr = {'function': {'name': 'objectNew', 'args': args}}
+        return [fn_expr, arg_text]
+
+    # Array creation?
+    if array_literals:
+        match_array_open = _R_EXPR_ARRAY_OPEN.match(expr_text)
+        if match_array_open:
+            arg_text = expr_text[len(match_array_open.group(0)):]
+            args = []
+            while True:
+                # Array close?
+                match_array_close = _R_EXPR_ARRAY_CLOSE.match(arg_text)
+                if match_array_close:
+                    arg_text = arg_text[len(match_array_close.group(0)):]
+                    break
+
+                # Array key separator
+                if args:
+                    match_array_separator = _R_EXPR_ARRAY_SEPARATOR.match(arg_text)
+                    if match_array_separator is None:
+                        raise BareScriptParserError('Syntax error', arg_text, 1, None, None)
+                    arg_text = arg_text[len(match_array_separator.group(0)):]
+
+                # Get the value
+                arg_value, arg_text = _parse_binary_expression(arg_text, None, array_literals)
+                args.append(arg_value)
+
+            fn_expr = {'function': {'name': 'arrayNew', 'args': args}}
+            return [fn_expr, arg_text]
+
     # Variable?
     match_variable = _R_EXPR_VARIABLE.match(expr_text)
     if match_variable:
@@ -593,13 +695,14 @@ def _parse_unary_expression(expr_text):
         return [expr, expr_text[len(match_variable.group(0)):]]
 
     # Variable (brackets)?
-    match_variable_ex = _R_EXPR_VARIABLE_EX.match(expr_text)
-    if match_variable_ex:
-        variable_name = _R_EXPR_VARIABLE_EX_ESCAPE.sub('\\1', match_variable_ex.group(1))
-        expr = {'variable': variable_name}
-        return [expr, expr_text[len(match_variable_ex.group(0)):]]
+    if not array_literals:
+        match_variable_ex = _R_EXPR_VARIABLE_EX.match(expr_text)
+        if match_variable_ex:
+            variable_name = _R_EXPR_VARIABLE_EX_ESCAPE.sub('\\1', match_variable_ex.group(1))
+            expr = {'variable': variable_name}
+            return [expr, expr_text[len(match_variable_ex.group(0)):]]
 
-    raise BareScriptParserError('Syntax error', expr_text)
+    raise BareScriptParserError('Syntax error', expr_text, 1, None, None)
 
 
 # BareScript expression regex
@@ -612,15 +715,41 @@ _R_EXPR_FUNCTION_CLOSE = re.compile(r'^\s*\)')
 _R_EXPR_GROUP_OPEN = re.compile(r'^\s*\(')
 _R_EXPR_GROUP_CLOSE = re.compile(r'^\s*\)')
 _R_EXPR_NUMBER = re.compile(r'^\s*(0x[A-Fa-f0-9]+|[+-]?\d+(?:\.\d*)?(?:e[+-]?\d+)?)')
+_R_EXPR_ARRAY_OPEN = re.compile(r'^\s*\[')
+_R_EXPR_ARRAY_SEPARATOR = re.compile(r'^\s*,')
+_R_EXPR_ARRAY_CLOSE = re.compile(r'^\s*\]')
+_R_EXPR_OBJECT_OPEN = re.compile(r'^\s*\{')
+_R_EXPR_OBJECT_SEPARATOR = re.compile(r'^\s*:')
+_R_EXPR_OBJECT_SEPARATOR2 = re.compile(r'^\s*,')
+_R_EXPR_OBJECT_CLOSE = re.compile(r'^\s*\}')
 _R_EXPR_STRING = re.compile(r"^\s*'((?:\\\\|\\'|[^'])*)'")
-_R_EXPR_STRING_ESCAPE = re.compile(r'\\([\\\'])')
 _R_EXPR_STRING_DOUBLE = re.compile(r'^\s*"((?:\\\\|\\"|[^"])*)"')
-_R_EXPR_STRING_DOUBLE_ESCAPE = re.compile(r'\\([\\"])')
 _R_EXPR_VARIABLE = re.compile(r'^\s*([A-Za-z_]\w*)')
 _R_EXPR_VARIABLE_EX = re.compile(r'^\s*\[\s*((?:\\\]|[^\]])+)\s*\]')
 _R_EXPR_VARIABLE_EX_ESCAPE = re.compile(r'\\([\\\]])')
 
 
+# String literal escapes
+_R_EXPR_STRING_ESCAPES = re.compile(r'\\([nrtbf\'"\\]|u[0-9a-fA-F]{4})')
+
+def _replace_string_escape(match):
+    esc = match.group(1)
+    if esc.startswith('u'):
+        return chr(int(esc[1:], 16))
+    return _EXPR_STRING_ESCAPES[esc]
+
+_EXPR_STRING_ESCAPES = {
+    'n': '\n',
+    'r': '\r',
+    't': '\t',
+    'b': '\b',
+    'f': '\f',
+    "'": "'",
+    '"': '"',
+    '\\': '\\'
+}
+
+
 class BareScriptParserError(Exception):
     """
     A BareScript parser exception
@@ -645,19 +774,24 @@ class BareScriptParserError(Exception):
 
        The error line number
 
+    .. attribute:: script_name
+       :type: str or None
+
+       The script name
+
     :param error: The error description
     :type error: str
     :param line: The line text
     :type line: str
     :param column_number: The error column number
-    :type column_number: int, optional
+    :type column_number: int
     :param line_number: The error line number
-    :type line_number: int or None, optional
-    :param prefix: The error message prefix line
-    :type prefix: str or None, optional
+    :type line_number: int or None
+    :param script_name: The script name
+    :type script_name: str or None
     """
 
-    def __init__(self, error, line, column_number=1, line_number=None, prefix=None):
+    def __init__(self, error, line, column_number, line_number, script_name):
         # Parser error constants
         line_length_max = 120
         line_suffix = ' ...'
@@ -679,9 +813,9 @@ class BareScriptParserError(Exception):
                 line_column -= line_left - len(line_prefix)
 
         # Format the message
-        newline = '\n'
+        error_prefix = f'{script_name or ""}:{line_number}: ' if line_number else ''
         message = f'''\
-{(prefix + newline) if prefix is not None else ''}{error}{(', line number ' + str(line_number)) if line_number is not None else ''}:
+{error_prefix}{error}
 {line_error}
 {' ' * (line_column - 1)}^
 '''
@@ -690,3 +824,4 @@ class BareScriptParserError(Exception):
         self.line = line
         self.column_number = column_number
         self.line_number = line_number
+        self.script_name = script_name
